import sys
import math
import random
import json


with open('heuristics_configuration.json', 'r') as f:
    data = json.load(f)

if data['temperature']:
   temperature_SA = (data['temperature'])

if data['mintemp']:
   mintemp_SA = (data['mintemp'])

if data['alpha']:
   alpha_SA = (data['alpha'])

if data['rep']:
   repet_SA = (data['rep'])

if data['dev']:
   dev_R2R = (data['dev'])

if data['maxi']:
   maxit_R2R = (data['maxi'])


class SimulatedAnnealing():
    def __init__(self):
        self.base_value = -1000000.0
        self.best_value = -1000000.0
        self.final_value = -1000000.0

        self.iterations_since_improvement = 0
         
    def configure(self, temperature, min_temperature, alpha, repetitions):
        self.temperature = temperature_SA
        self.repetitions = mintemp_SA
        self.alpha = alpha_SA
        self.min_temperature =  mintemp_SA
        
    def set_base_solution(self, solution):
        solution_value = solution.compute_value()
        
        self.base_value = solution_value
        self.base_solution_json = solution.to_json()

        if self.base_value > self.best_value:
            self.final_value = self.base_value
            self.final_solution_json = self.base_solution_json

            self.best_value = self.base_value
            self.iterations_since_improvement = 0
        else:
            self.iterations_since_improvement += 1

    def continue_solving(self, iterations):
        if iterations % self.repetitions == 0:
            self.temperature = self.temperature * self.alpha
        
        continue_solving = True

        if self.temperature < self.min_temperature:
            print("Reached minimum temperature {}".format(iterations))
            continue_solving = False
        elif iterations > 150000 and self.iterations_since_improvement >= 10000:
            print("Have not improved in 10000 iterations {}".format(iterations))
            continue_solving = False

        return continue_solving
    
    def accept_solution(self, neighbor_solution):
        neighbor_solution_value = neighbor_solution.compute_value()
        
        value_delta = (self.base_value - neighbor_solution_value) / (abs(self.base_value) + abs(neighbor_solution_value))

        try:
            accept_probability =  1.0 / math.exp((value_delta / self.temperature))
        except OverflowError:
            accept_probability = 0
        except ZeroDivisionError:
            accept_probability = 1
            

        return random.random() < accept_probability
    
           
class RecordToRecord():
    def __init__(self):
        self.base_value = -1000000.0
        self.best_value = -1000000.0
        self.final_value = -1000000.0

        self.iterations_since_improvement = 0

    def configure(self, deviation, max_iterations):           
        self.deviation = dev_R2R
        self.max_iterations = maxit_R2R        
    
    def set_base_solution(self, solution):
        solution_value = solution.compute_value()
        self.base_value = solution_value
        
        if solution_value > self.best_value:
            self.best_value = solution_value
            self.best_solution_json = solution.to_json()

            self.final_value = self.best_value
            self.final_solution_json = self.best_solution_json

            self.iterations_since_improvement = 0
        else:
            self.iterations_since_improvement += 1

    def continue_solving(self, iterations):
        continue_solving = True

        if iterations > maxit_R2R:
            print("Reached maximum iterations {}".format(iterations))
            continue_solving = False
        elif iterations > 150000 and self.iterations_since_improvement >= 10000:
            print("Have not improved in 10000 iterations {}".format(iterations))
            continue_solving = False

        return continue_solving
        
    def accept_solution(self, solution):
        solution_value = solution.compute_value()

        return solution_value > self.best_value - abs(self.best_value * dev_R2R)
    
   
